use strict;
use warnings;

# ***************************************************************************
# *   Copyright (C) 2010 Matti Kankainen                                    *
# *   matti.kankainen@helsinki.fi                                           *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU General Public License as published by  *
# *   the Free Software Foundation; either version 2 of the License, or     *
# *   (at your option) any later version.                                   *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU General Public License for more details.                          *
# *                                                                         *
# *   You should have received a copy of the GNU General Public License     *
# *   along with this program; if not, write to the                         *
# *   Free Software Foundation, Inc.,                                       *
# *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
# ***************************************************************************/

#########################################################################################################
#read parameters
sub golm_mapper {
	my($file, $column, $winwidth, $outputf, $param)	= @_;
	my($line)					= "";
	my(@cols)					= ();
	my($ri)						= "";
	my($spec)					= "";
	my($name)					= "";
	my($counter2)					= 0;
	my($pid)					= 0;
	my($pid2)					= 0;
	my($analyte_line)				= "";
	my($metabol_line)				= "";
	my($goml_line)					= "";
	my(%goml_res)					= ();
	my($pos)					= 0;
	my(@goml_ar)					= ();
	my($goml_value)					= "";
	my($print_me)					= 0;
	my($print_line1)				= "";
	my($print_line2)				= "";
	my($print_line)					= "";
	my($metabol_name)				= "";
	my($html_base)					= "http://gmd.mpimp-golm.mpg.de";
	my($print_meta)					= "";
	my(@meta_cols)					= ();
	my($synonyms)					= "";
	my($kegg_compound)				= "";
	my($kegg_temp_cpd)				= "";
	my($all_kegg_compound)				= "";
	my($temp_synonym)				= "";
	my($rnd_file)					= "";

	$rnd_file = $outputf;
	open(OUTFILE, ">$rnd_file");

	#a file in which name is first, RI is in second and spectra in the third column
	open(IN, $file) or die "file $file not found\n";
	while($line = <IN>) {
		$line 	=~ s/^\s+|\s+$//g;
		if($line =~ m/^#/) { next; }

		@cols	= split(/\t/, $line);
		if(@cols < 2) { next; }

		$name	= $cols[0];
		$name	=~ s/^.*_//g;
		$name	=~ s/^\*//g;
		$name	=~ s/^X\.//g;
		$ri	= $cols[1];
		$spec	= $cols[2];
		$spec	=~ s/[\[\]]//gi;
		$spec 	=~ s/^\s+|\s+$//g;
		if($spec =~ m/[,:]/) { 
			$spec	=~ s/\s+//g;
			$spec	=~ s/[,:]/+/g;
		} else {
			$spec   =~ s/\s+/+/g;
		}
		$pos	= -2;

		#print STDERR "wget '$html_base/webservices/wsLibrarySearch.asmx/LibrarySearch?ri=$ri&riWindow=$winwidth&AlkaneRetentionIndexGcColumnComposition=$column&spectrum=$spec' -o $rnd_file.log -O $rnd_file.output\n\n";

		$pid = system("wget '$html_base/webservices/wsLibrarySearch.asmx/LibrarySearch?ri=$ri&riWindow=$winwidth&AlkaneRetentionIndexGcColumnComposition=$column&spectrum=$spec' -o $rnd_file.log -O $rnd_file.output");
		%goml_res = ();

		if($pid == 0) {
			open(GOML, "$rnd_file.output") or die "output not created\n";
			while($goml_line = <GOML>) {
				$goml_line 	=~ s/^\s+|\s+$//g;
				$goml_value	= $goml_line;
				$goml_value	=~ s/^\<.*?\>//; #remove preceding tag
				$goml_value	=~ s/\<.*?\>$//; #remove the end tag
				$pos		= -1;

				if($goml_line =~ m/\<Results\>/)		{ 
					if($pos <= -2) { die "end tag not found\n"; } 
					@goml_ar = (0,0,0,0,0,0,0,0,0); 
					$pos = -1; 
				} elsif($goml_line =~ m/\<analyteID\>/)			{ $pos = 0; $goml_ar[$pos] = $goml_value; }
				elsif($goml_line =~ m/\<ri\>/)				{ $pos = 1; $goml_ar[$pos] = $goml_value; }
				elsif($goml_line =~ m/\<riDiscrepancy\>/)		{ $pos = 2; $goml_ar[$pos] = $goml_value; }
				elsif($goml_line =~ m/\<DotproductDistance\>/)		{ $pos = 3; $goml_ar[$pos] = $goml_value; }
				elsif($goml_line =~ m/\<EuclideanDistance\>/)		{ $pos = 4; $goml_ar[$pos] = $goml_value; }
				elsif($goml_line =~ m/\<HammingDistance\>/)		{ $pos = 5; $goml_ar[$pos] = $goml_value; }
				elsif($goml_line =~ m/\<JaccardDistance\>/)		{ $pos = 6; $goml_ar[$pos] = $goml_value; }
				elsif($goml_line =~ m/\<s12GowerLegendreDistance\>/)	{ $pos = 7; $goml_ar[$pos] = $goml_value; }	#modified --> modify everywhere
				elsif($goml_line =~ m/\<analyteName\>/)			{ $pos = 8; $goml_ar[$pos] = $goml_value; }
				elsif($goml_line =~ m/\<\/Results\>/)			{ $pos = -2; }					#is the last line (modified)
				else 							{ $pos = -3; } 

				if($pos == -2 && $goml_ar[8] ne 'NA') {	
					#default parameters
					if($goml_ar[3] <= @$param[0] && $goml_ar[4] <= @$param[1] && $goml_ar[5] <= @$param[2] && $goml_ar[6] <= @$param[3] && $goml_ar[7] <= @$param[4]) {
						#print STDERR "$goml_ar[3] <= @$param[0] && $goml_ar[4] <= @$param[1] && $goml_ar[5] <= @$param[2] && $goml_ar[6] <= @$param[3] && $goml_ar[7] <= @$param[4]\n";

						if(!(exists($goml_res{$goml_ar[8]}))) {
							$goml_res{$goml_ar[8]}{'text'} 	= join("\t", @goml_ar);
							$goml_res{$goml_ar[8]}{'val'}	= $goml_ar[4];
							$goml_res{$goml_ar[8]}{'test'}	= (($goml_ar[3]+$goml_ar[4])/2) + (($goml_ar[6]+$goml_ar[7])/2);
						} elsif($goml_res{$goml_ar[8]}{'test'} <= (($goml_ar[3]+$goml_ar[4])/2) + (($goml_ar[6]+$goml_ar[7])/2)) {
							$goml_res{$goml_ar[8]}{'text'} 	= join("\t", @goml_ar);
							$goml_res{$goml_ar[8]}{'val'} 	= $goml_ar[4];
							$goml_res{$goml_ar[8]}{'test'}	= (($goml_ar[3]+$goml_ar[4])/2) + (($goml_ar[6]+$goml_ar[7])/2);
						}
					}
				}
			}
			close GOML;
		}
		
		for my $key (sort { $goml_res{$a}{'val'} <=> $goml_res{$b}{'val'}; } (keys(%goml_res))) {
			@cols = split("\t", $goml_res{$key}{'text'});
			$pid = system("wget '$html_base/Analytes/$cols[0].aspx' -o $rnd_file.log -O $rnd_file.analyte");

			if($pid == 0) {
				$print_me 	= 0;
				$print_meta	= "";
				$kegg_compound	= "";
				open(ANAL, "$rnd_file.analyte") or die "analyte not found\n";
				while($analyte_line = <ANAL>) {
					$analyte_line =~ s/^\s+|\s+$//g;
					if($analyte_line =~ m/gvCompoundMapping/) 	{ $print_me = 1; }
					if($analyte_line =~ m/Spectra/) 		{ $print_me = 0; }
					if($print_me == 1) {
						if($analyte_line =~ m/\/Metabolites\//i) { 
							$analyte_line 	=~ s/\.aspx.*$//g;
							$metabol_name	= $&;
							$metabol_name	=~ s/.*self\"\>//g;
							$metabol_name	=~ s/\<.*$//g;
							$analyte_line 	=~ s/^.*\/Metabolites\///g;
							$synonyms	= "$metabol_name|";

							#$print_meta 	.= "$metabol_name\t$analyte_line.aspx"; 
							$print_meta 	.= "$metabol_name";

							$pid2 = system("wget '$html_base/Metabolites/$analyte_line.aspx' -o $rnd_file.log -O $rnd_file.metabolite");
							if($pid2 == 0) {
								open(META, "$rnd_file.metabolite") or die "metabol not found\n";
								while($metabol_line = <META>) {
									while($metabol_line =~ m/KEGG.*?,/gi) {
										$kegg_temp_cpd 	= $&;
										$kegg_temp_cpd 	=~ s/,//g;
										$kegg_temp_cpd 	=~ s/^.*cpd://g;
										if($kegg_temp_cpd =~ m/not found/) { $kegg_temp_cpd = "not_found"; }
 
										$kegg_compound 	.= "\t$kegg_temp_cpd";
										$print_meta	.= "\t$kegg_temp_cpd";
									}
				
									#if($metabol_line =~ m/KEGG/gi && $metabol_line =~ m/jp/gi) {
									#	$metabol_line	=~ s/^.*self\"\>//g;
									#	$metabol_line	=~ s/\<.*$//g;
									#	$metabol_line	=~ s/^\s+|\s+$//g;
									#	if($metabol_line =~ m/not found/) { $metabol_line = "not_found"; }
									#	$print_meta 	.= "\t$metabol_line";
									#	$kegg_compound	.= "\t$metabol_line";
									#}
									if($metabol_line =~ m/\<td\>synonym/) {
										$metabol_line	=~ s/^.*self\"\>//g;
										$metabol_line	=~ s/\<.*$//g;
										$metabol_line	=~ s/^\s+|\s+$//g;
										$synonyms	.= "$metabol_line|";
									}
								}
							}
							$synonyms	 =~ s/\|$//g;
							$print_meta 	.= "\t".lc($synonyms)."\n";
						
							#$kegg_compound  .= ";"	#use this to separate metabolites
						}
					}
				}
				close ANAL;
				$print_meta		=~ s/\s+$//g;
				$print_meta		=~ s/\t+/\t/g;
				$goml_res{$key}{'res'}	= "$print_meta";

				$kegg_compound		=~ s/not_found//g;
				$kegg_compound		=~ s/,+/,/g;
				$goml_res{$key}{'kegg'}	.= $kegg_compound;
				$all_kegg_compound 	.= "$kegg_compound";

				@meta_cols		= split(/\|/, $synonyms);
				push(@meta_cols, "$key");

				for(my($i) = 0; $i < @meta_cols; ++$i) {
					$temp_synonym = $meta_cols[$i];
					$temp_synonym =~ s/^\s+|\s+$//g;
					if($temp_synonym =~ m/\Q$name\E/) { $goml_res{$key}{'val'} = -100; }
				}
			}
		}
		
		$counter2 		= 0;
		$print_line1 		= "";
		$print_line2		= "";
		#$print_line = "$name\t$all_kegg_compound\n";

		#print STDERR "$print_line1\n";
		for my $key (sort { $goml_res{$a}{'val'} <=> $goml_res{$b}{'val'}; } (keys(%goml_res))) {
			@cols 			= split("\t", $goml_res{$key}{'text'});
			$all_kegg_compound 	= $goml_res{$key}{'kegg'};
			$all_kegg_compound	=~ s/^\s+|\s+$//g;
			$all_kegg_compound	=~ s/\t/,/g;
			$goml_res{$key}{'res'}	=~ s/\n/\n\t\t#/gi;

			#$print_line1 		=~ s/^\|+//g;
			$print_line1 		.= "$all_kegg_compound|";

			$print_line2 		.= "\t#$cols[8]\t$cols[0].aspx\n";
			$print_line2 		.= "\t\t#$goml_res{$key}{'res'}\n";
			++$counter2;

			#if name is given and it is identical, escape the loop and do not report more hits
			#the idea of this is that only one hit is reported for manually verified analytes
			if($goml_res{$key}{'val'} == -100) { last; }
		}
		#if($counter == 5) { die; }
		$print_line1 =~ s/^\s+|\s+$//g;
		$print_line1 =~ s/\|+/\|/g;
		$print_line1 =~ s/,+/,/g;
		$print_line1 =~ s/^\|+|\|+$//g;
		$print_line2 =~ s/^\s+|\s+$//g;
		$print_line  = "$name\t$print_line1\n\t$print_line2\n";
		$print_line  =~ s/^\s+|\s+$//g;
		print OUTFILE "$print_line\n";
	}

	close OUTFILE;
	if(-e "$rnd_file.analyte") { system("rm $rnd_file.analyte"); }
	if(-e "$rnd_file.metabolite") { system("rm $rnd_file.metabolite"); }
	if(-e "$rnd_file.output") { system("rm $rnd_file.output"); }
	if(-e "$rnd_file.log") { system("rm $rnd_file.log"); }

}
#########################################################################################################
#
#example
#system("wget 'http://gmd.mpimp-golm.mpg.de/webservices/wsLibrarySearch.asmx/LibrarySearch?ri=2224.70&riWindow=5&AlkaneRetentionIndexGcColumnComposition=VAR5&spectrum=75 998 73 847 55 589 117 533 129 430 67 232 69 196 45 189 81 184 96 170 57 149 84 148 145 148 83 133 54 122 98 119 56 117 95 116 97 106 74 99 47 96 79 92 131 89 82 86 132 86 61 83 68 74 76 72 77 72 53 69 339 61 109 58 110 57 70 54 116 54 130 54 118 52 93 46' -o out1 -O out2");

#Before Aug 2011
#  <AnnotatedMatch>
#    <analyteID>34cef23b-ca62-4f92-a4ff-006a68a01592</analyteID>
#    <ri>2225.44067</ri>
#    <riDiscrepancy>0.7407349</riDiscrepancy>
#    <DotproductDistance>0.00622342667</DotproductDistance>
#    <EuclideanDistance>0.0133346</EuclideanDistance>
#    <HammingDistance>32</HammingDistance>
#    <JaccardDistance>0.45714286</JaccardDistance>
#    <BinaryDistance>0.5130415</BinaryDistance>
#    <analyteName>Octadecenoic acid, 9-(E)- (1TMS)</analyteName>
#  </AnnotatedMatch>

#From Aug 2011
#  <Results>
#    <spectrumID>ca1daf1f-a4bc-4129-b34b-c4f7943efc77</spectrumID>
#    <analyteID>69726c71-7238-4fdc-85fb-08c4af1e77b5</analyteID>
#    <ri>1924.07117</ri>
#    <riDiscrepancy>17.5926666</riDiscrepancy>
#    <DotproductDistance>0.0216349978</DotproductDistance>
#    <EuclideanDistance>0.008254799</EuclideanDistance>
#    <HammingDistance>618</HammingDistance>
#    <JaccardDistance>0.973228335</JaccardDistance>
#    <s12GowerLegendreDistance>0.9145379</s12GowerLegendreDistance>
#    <spectrumName>Galactitol (6TMS) [A194001-ambient-na-6]</spectrumName>
#    <analyteName>Galactitol (6TMS)</analyteName>
#    <metaboliteID>118ffb2b-9cdf-4cf4-9706-6865b46ba004</metaboliteID>
#  </Results>

1;
