use strict;
use warnings;
use FindBin qw($Bin);

# ***************************************************************************
# *   Copyright (C) 2010 Matti Kankainen                                    *
# *   matti.kankainen@helsinki.fi                                           *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU General Public License as published by  *
# *   the Free Software Foundation; either version 2 of the License, or     *
# *   (at your option) any later version.                                   *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU General Public License for more details.                          *
# *                                                                         *
# *   You should have received a copy of the GNU General Public License     *
# *   along with this program; if not, write to the                         *
# *   Free Software Foundation, Inc.,                                       *
# *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
# ***************************************************************************/

#########################################################################################################
#read parameters
sub kegg_enrichment {
	my($name_file, $mapping_file, $organism, $filter_file, $output, $mode, $list_size, 
	   $kmode, $zero, $direction, $perm_num, $kegg_figures, $column, $mink, $kegg_userbck, $no_orginfo)		= @_;
	my($line)													= "";
	my(@cols)													= ();
	my(%maps)													= ();
	my(%org_maps)													= ();
	my(%background_map)												= ();
	my(%userbackground)												= ();
	my(%filter)													= ();
	my(%compound)													= ();
	my(@map_cols)													= ();
	my(@spec_map_file)												= ();
	my(@temp_spec_map_file)												= ();
	my($do_kegg_map)												= 0;
	my($rnd_element)												= 0;
	my($print_output)												= 0;
	my(%pval_table)													= ();
	my(@pval_table_all)												= ();
	my(@original_array)												= ();
	my(@original_array_all)												= ();
	
	$maps{"mapall"}{'N'} = 0;
	$maps{"mapall"}{'n'} = 0;

	####################################################################
        #print parameters
	open(OUT, ">$output") or die "$filter_file not open for print\n";
	print OUT "\n\#input file=$filter_file\n";
	print OUT "\#direction=$direction\n";
	print OUT "\#sort columns=$column\n";
	print OUT "\#permutations=".($perm_num-1)."\n";
	print OUT "\#kselection=$kmode\n";
	print OUT "\#penalty_mode=$zero\n";
	print OUT "\#minimum k=$mink\n";
	print OUT "\#list_size=$list_size\n";
	print OUT "\#organism=$organism\n";
	print OUT "\#use_compounds_without_organism=$no_orginfo\n";
	print OUT "\#background_file=$kegg_userbck\n\n";
	close OUT;

	#retrieve maps that are sepcific to the given organism(s) set
	open(IN, $mapping_file) or die "cpd_map_org: $mapping_file not found\n";
	while($line = <IN>) {
		@cols   = split(/\t/, $line);
		if(!($line =~ m/map/gi)) { next; }
		if( $cols[2] =~ m/$organism/i || $organism eq 'all' || (length($cols[2]) == 0 && $no_orginfo == 1) ) {
			@map_cols       = split(/\,/, $cols[3]);
			for(my($i) = 0; $i < @map_cols; ++$i) {
				$map_cols[$i] =~ s/^\s+|\s+$//g;
				if(!exists($org_maps{$map_cols[$i]})) {
					$org_maps{$map_cols[$i]} = 1;
				} else {
					$org_maps{$map_cols[$i]} = $org_maps{$map_cols[$i]} + 1;
				}
			}
		}
	}
	close IN;

	####################################################################
	#file having the names of all maps
	#normally this file is map_title.tab
	open(IN, $name_file) or die "map-names: $name_file not found\n";
	while($line = <IN>) {
		$line 				=~ s/^\s+|\s+$//g;
		@cols				= split(/\t+/, $line);

		if($cols[0] =~ m/SMP/) {
			$maps{"$cols[0]"}{'name'}    = $cols[1];
			$maps{"$cols[0]"}{'k'}       = 0;
			$maps{"$cols[0]"}{'K'}       = 0;
		} else { 
			$maps{"map$cols[0]"}{'name'} 	= $cols[1];
			$maps{"map$cols[0]"}{'k'} 	= 0;
			$maps{"map$cols[0]"}{'K'} 	= 0;
		}
	}
	close IN;

	####################################################################
	#read user specific background compounds
	#is a file is provided, only compounds listed in it all considered
	#as valid and read into the system at the next phase
	#
	#this works also for *.map files if it is not treated as
	#all KEGG compounds exists in separate tab-columns in rows having
	#the '#'-char

	if($kegg_userbck ne 'null') {
		if(-e $kegg_userbck) {
			open(IN, $kegg_userbck);
			while($line = <IN>) {
				$line =~ s/^\s+|\s+$//g;
				if(length($line) > 0) {
					#supprots MPEA-outputs
					@cols = split(/\t|\||,/, $line);
					#@cols = split(/\t/, $line);
					for(my($i) = 0; $i < @cols; ++$i) {
						$userbackground{$cols[$i]} = 1;
					}
				}
			}
			close IN;
		}
	}
	#if no compounds exists, include all compounds
	if(scalar(keys(%userbackground)) == 0) {
		open(IN, $mapping_file) or die "cpd_map_org: $mapping_file not found\n";
		while($line = <IN>) {
			$line	=~ s/^\s+|\s+$//g;
			@cols   = split(/\t/, $line);
			$userbackground{$cols[0]} = 1;
		}
		close IN;
	}

	####################################################################
	#file containing information about enzymes, compounds, maps and organism having the gene
	#normally this file is cpd_map_org.tab
	open(IN, $mapping_file) or die "cpd_map_org: $mapping_file not found\n";
	while($line = <IN>) {
		@cols	= split(/\t/, $line);
		if(!($line =~ m/map/gi)) { next; }

		#if((length($cols[2]) == 0) || ((length($cols[2]) > 0)) && ($cols[2] =~ m/$organism/i)) {
		if( $cols[2] =~ m/$organism/i || $organism eq 'all' || (length($cols[2]) == 0 && $no_orginfo == 1) ) {
			#maps which the compound belongs
			@map_cols 	= split(/\,/, $cols[3]);
			for(my($i) = 0; $i < @map_cols; ++$i) {
				$map_cols[$i] =~ s/^\s+|\s+$//g;
				if(length($map_cols[$i]) > 0) {
					if(exists($userbackground{$cols[0]})) {
						$background_map{$map_cols[$i]}{$cols[0]}	= 0;
						#cols[0] = compound | map_cols[$i] = maps which the compound belongs to
						$compound{$cols[0]}{$map_cols[$i]}	 	= 0;
					}
				}
			}
		}
	}
	close IN;

	$maps{"mapall"}{'N'} = scalar(keys(%compound));
	for my $key (keys( %background_map )) {
		$maps{$key}{'K'} = scalar(keys(%{$background_map{$key}}));
	}
	
	my($counter) = 0;
	####################################################################
	#read spectral mapping file, if needed turn the file around
	open(IN, $filter_file) or die "$filter_file not found\n";
        while($line = <IN>) {
                $line  =~ s/^\s+|\s+$//g;
                if(!($line =~ m/^#/)) {
			#if direction == 0, order is not changed
			if($direction == 0){
				push(@original_array_all, $line);
			#id direction == 1 reverse order
			} else {
				unshift(@original_array_all, $line);
			}
		}
	}
	close IN;

	#choose only certain amount of rows, if desired
	if($list_size == 0) {
		@original_array = @original_array_all;
	} else {
		if($list_size > scalar(@original_array_all)) { $list_size = scalar(@original_array_all); }
		@original_array = splice(@original_array_all, 0, $list_size);
	}

	####################################################################
        #calculate n, k
	for(my($i) = 0; $i < $perm_num; ++$i) {
		if($i < ($perm_num-1)) {
			@temp_spec_map_file	= @original_array_all; 	#use all metabolites 
			#@temp_spec_map_file 	= @original_array;	#use only the ones that were wished for to be analysed
			@spec_map_file		= ();
			while(scalar(@temp_spec_map_file) > 0) {
				$rnd_element = int(rand(scalar(@temp_spec_map_file)));
				push(@spec_map_file, $temp_spec_map_file[$rnd_element]);
				splice(@temp_spec_map_file, $rnd_element, 1);
			}
			if($list_size == 0) {
				@spec_map_file = @spec_map_file;
			} else {
				@spec_map_file = splice(@spec_map_file, 0, $list_size);
			}
			$do_kegg_map	= 0;
			$print_output	= 0;
		} else {
			@spec_map_file 	= @original_array;
			$do_kegg_map	= $kegg_figures;
			$print_output	= 1;
		}

		if($mode == 0) { #normal mode
			open(OUT, ">$output.temp") or die "$output.temp not created\n";
			&calculate_frequency(\@spec_map_file, \%compound, \%maps, $mode, $kmode, $zero); 
			&hyp_geom_dist_file(\%maps, \%org_maps, $mink);
			close OUT;
			&print_results("$output.temp", $output, $do_kegg_map, \%pval_table, \@pval_table_all, ($perm_num-1), $print_output);
		} else { #calculate n,k in iterative fashion
			open(OUT, ">$output.temp") or die "$output.temp not created\n";
			@temp_spec_map_file = ();
			for(my($i) = 0; $i < @spec_map_file; ++$i) {
				$line 	= $spec_map_file[$i];
				@cols	= split(/\t+/, $line);	
				push(@temp_spec_map_file, $spec_map_file[$i]);
				if(@cols > 1) {
					&calculate_frequency(\@temp_spec_map_file, \%compound, \%maps, $mode, $kmode, $zero);
					&hyp_geom_dist_file(\%maps, \%org_maps, $mink);
				}
			}
			close OUT;
			&print_results("$output.temp", $output, $do_kegg_map, \%pval_table, \@pval_table_all, ($perm_num-1), $print_output);
		}
	}
	if(-e("$output.temp")) { unlink("$output.temp"); }
};
#########################################################################################################
#print_results
sub print_results {
	my($filter_file, $output, $get_kegg_map, $pval_pointer, $pval_pointer_all, $perm_num, $do_print)	= @_;
	my($line)									= "";
	my(@cols)									= ();
	my(%results)									= ();
	my(@temp_cols)									= ();
	my($kegg_phrase)								= "";
	my($kegg_figure)								= "";
	my($ko_map)									= "";
	my($pid)									= 0;
	my($min_val)									= 0;
	my($hypdistout)									= "$output.hyps";
	my($min_pval)									= 1;

	$pid = system("$Bin/fast_hyp_cdf $filter_file $hypdistout 5:4:3:2");
	#no check as this returns wierd numbers!!!

	open(IN, "$hypdistout") or die "$hypdistout (temp) not opened\n";
	while($line = <IN>) {		
		$line 		=~ s/^\s+|\s+$//g;
		@cols 		= split(/\t/, $line);
		$line		=~ s/\t\#.*$//g;

		if(!exists($results{$cols[0]})) {
			$results{$cols[0]}{'name'}	= $cols[1];
			$results{$cols[0]}{'N'}		= $cols[2];
			$results{$cols[0]}{'K'}		= $cols[3];
			$results{$cols[0]}{'n'}		= $cols[4];
			$results{$cols[0]}{'k'}		= $cols[5];
			$results{$cols[0]}{'nall'}	= $cols[6];
			$results{$cols[0]}{'comp'}	= $cols[7];
			$results{$cols[0]}{'pval'} 	= $cols[8];
		} else {
			if($results{$cols[0]}{'pval'} > $cols[8]) {
                        	$results{$cols[0]}{'name'}	= $cols[1];
                        	$results{$cols[0]}{'N'}		= $cols[2];
                        	$results{$cols[0]}{'K'}		= $cols[3];
                        	$results{$cols[0]}{'n'}		= $cols[4];
                        	$results{$cols[0]}{'k'}		= $cols[5];
                        	$results{$cols[0]}{'nall'}	= $cols[6];
                        	$results{$cols[0]}{'comp'}	= $cols[7];
                        	$results{$cols[0]}{'pval'}	= $cols[8];
			}
		}
	}
	close IN;
	#unlink($hypdistout) or die "$hypdistout not removed\n";

	if($do_print == 0) {
		for my $key (sort { $results{$a}{'pval'} <=> $results{$b}{'pval'} ;} (keys(%results))) {
			if($results{$key}{'pval'} < $min_pval) {
				$min_pval = $results{$key}{'pval'};
			}

			if(!(exists(${$pval_pointer}{$key}))) {
				${$pval_pointer}{$key} = [$results{$key}{'pval'}];
			} else {
				push(@{${$pval_pointer}{$key}}, $results{$key}{'pval'});
			}
		}
		#print STDERR "$min_pval\n";
		push(@$pval_pointer_all, $min_pval);
	} else {
		open(OUT, ">>$output");
		for my $key (sort { $results{$a}{'pval'} <=> $results{$b}{'pval'} ;} (keys(%results))) {
			#prints the name, N (kegg compounds), K (kegg compound of the pathway)
			#n/nall = list position including those with KEGG-ID or all compounds
			#k = the chosen k (min, max, median appraoch)
			#p-value and bootstrap p-value
			#all compounds to be searched from KEGG

			print OUT "$key\t$results{$key}{'name'}\t$results{$key}{'N'}\t$results{$key}{'K'}\t";
			print OUT "$results{$key}{'n'}/($results{$key}{'nall'})\t$results{$key}{'k'}\t";
			print OUT "$results{$key}{'pval'}\t";

			$min_val = 0;
			if(exists(${$pval_pointer}{$key})) {
				for(my($j) = 0; $j < scalar(@{${$pval_pointer}{$key}}); ++$j) {
					if(@{${$pval_pointer}{$key}}[$j] <= $results{$key}{'pval'}) { ++$min_val; }
				}
			}
		
			if($perm_num == 0) { print OUT "1.0000"; }
			else { print OUT "".sprintf("%.4f", ($min_val/$perm_num)).""; }

			$min_pval = 0;
			for(my($i) = 0; $i < @$pval_pointer_all; ++$i) {
				if(@$pval_pointer_all[$i] <= $results{$key}{'pval'}) { ++$min_pval; }
			}

			if($perm_num == 0) { print OUT "\t1.0000"; }
			else { print OUT "\t".sprintf("%.4f", ($min_pval/$perm_num)).""; }

			$results{$key}{'comp'} =~ s/\,+$//g;
			print OUT "\t$results{$key}{'comp'}\n";

			#########################################################################################################
			#do KEGG figures

			if(($get_kegg_map == 1) && (!($key =~ m/SMP/i))) {
				$kegg_phrase 	= $results{$key}{'comp'};
				$kegg_phrase 	=~ s/^\#//g;
				$kegg_phrase 	=~ s/\,/\/cpd\:/g;
				$kegg_phrase 	= "\/cpd:$kegg_phrase";
				$ko_map		= $key;
				$ko_map		=~ s/map/ko/g;

				$pid = system("wget \"http://www.genome.jp/kegg-bin/show_pathway?$ko_map/default%3dred$kegg_phrase\" -O $output\_$ko_map.html -o $output\_kegg.log");
				if($pid != 0) { die "kegg-color has caused a problem $pid\n";}
				open(KEGG, "$output\_$ko_map.html") or die "kegg.html file $output\_$ko_map.html not found\n";
				while($line = <KEGG>) {
					if($line =~ m/mark_pathway/) {
						$kegg_figure = $line;
						$kegg_figure =~ s/^.*mark_pathway//g;
						$kegg_figure =~ s/png.*$/png/g;
						$kegg_figure =~ s/^\s+|\s+$//g;
						#$kegg_figure =~ s/map/ko/g;
					}
				}
				close KEGG;
		
				$pid = system("wget \"http://www.genome.jp/tmp/mark_pathway$kegg_figure\" -O $output\_$ko_map.png -o $output\_kegg.log");
				if($pid != 0) { die "kegg retrieve has caused a problem $pid\n"; }
				$pid = system("rm $output\_$ko_map.html");
				if($pid != 0) { die "rm has caused a problem $pid\n"; }
				$pid = system("rm $output\_kegg.log");
				if($pid != 0) { die "rm has caused a problem $pid\n"; }
			}
		}
	}
	close OUT;
};
#########################################################################################################
#run hyp.geom.dist
sub hyp_geom_dist_file {
	my($maps, $org_maps, $mink)					= @_;
	
	for my $key ( sort (keys ( %{$maps} ))) {
		if(exists(${$org_maps}{$key})) {
			if(exists(${$maps}{$key}{'k'}) && (${$maps}{$key}{'k'} >= $mink)) {
				if(!(exists(${$maps}{$key}{'name'}))) { ${$maps}{$key}{'name'} = "unknown"; }
				print OUT "$key\t${$maps}{$key}{'name'}\t${$maps}{'mapall'}{'N'}\t";
				print OUT "${$maps}{$key}{'K'}\t${$maps}{'mapall'}{'n'}\t${$maps}{$key}{'k'}\t";
				print OUT "${$maps}{'mapall'}{'nall'}\t#${$maps}{$key}{'compound'}\n";
			}
		}
	}
};
#########################################################################################################
#read compound-indetifiers resolved using goml-mapper
sub calculate_frequency {
	my($file, $compounds, $maps, $mode, $kmode, $zero)		= @_;
	my($line)							= "";
	my(@cols)							= ();
	my(@all_analytes)						= ();
	my(@this_metabols)						= ();
	my($print_line)							= "";
	my(%analyte_maps)						= ();
	my(%all_maps)							= ();
	my($this_maps)							= "";
	my(%compounds_for_maps)						= ();
	my(%map_compound_meta)						= ();
	my(%nfreq)							= ();
	my(%this_map_compound_meta)					= ();
	my(@this_map_keys)						= ();
	my(@pathlist)							= ();
	my(%freqlist)							= ();
	my(@freqarray)							= ();
	my(@num_compound)						= ();
	${$maps}{"mapall"}{'n'}						= 0;
	${$maps}{"mapall"}{'nall'}					= 0;
	my($counter)							= 0;
	my($temp_cmp)							= 0;
	my($analyte_counter)						= 0;
	my(%analyte_freq)						= ();
	my($usek)							= 0;
	my($zero_counter)						= 0;

	for my $key (keys( %{$maps})) {
		${$maps}{$key}{'k'} = 0;
	}
	
	#loop over identified spectrums
	# C02076|C01697,C00392|C06466|C06465,C01487|C00031,C00221,C00267,C00293,C00293,C01487|C06467
	#(where | speparates analytes and , separates alternative metabolites/KEGG-ids)
	for(my($h) = 0; $h < @$file; ++$h) {
		$line 		= @$file[$h];
		@cols  		= split(/\t+/, $line);
		if(@cols > 1) {
			#$cols[1]		=~ s/\|.*$//g;
			#modify this to separate ";", if so metabolites are wished to be separated (modify also golm_mapper.pm)
			@all_analytes		= split(/[\|]/, $cols[1]);
			$print_line		= "$line\n";
			%all_maps		= ();
			%analyte_freq		= ();
			%analyte_maps		= ();
			$temp_cmp		= "compound".$counter;
			$analyte_counter	= 0;

			#loop over alternative analytes/matches
			for(my($i) = 0; $i < @all_analytes; ++$i) {
				@this_metabols 		= split(/\,/, $all_analytes[$i]);
				$this_maps		= "";
				++$analyte_counter;

				#loop over alternative metabolites and their KEGG-ids
				for(my($j) = 0; $j < @this_metabols; ++$j) {
					#check if the KEGG-id can be associated to some pathway
					if(exists(${$compounds}{$this_metabols[$j]})) {
						$this_maps	= "";

						for my $key (keys (%{${$compounds}{$this_metabols[$j]}} )) { 
							if(!(exists($analyte_maps{$key}{"$temp_cmp.$i"}))) 	{ $analyte_maps{$key}{"$temp_cmp.$i"} = 1; }
							if(!(exists($analyte_freq{"$temp_cmp.$i"}))) 		{ $analyte_freq{"$temp_cmp.$i"} = 1; }

							$all_maps{$key}		= 0;
							$this_maps 		.= "$key,";

							#key = map, temp_cmp = row, this_metabols[j] = KEGGid
							$map_compound_meta{$key}{$temp_cmp}{$this_metabols[$j]} = 1;
							$compounds_for_maps{$key}{$this_metabols[$j]} 		= 1;
						}

						$print_line 			.= "\t$analyte_counter:$this_metabols[$j]:$this_maps\n"
					}
				}
			}

			#dummy-variables are not added, if at least one of the identifiers within the "interchangeable" group
			#can be linked to the pathway. This check up is done for each "interchangeable" group and as many times
			#as the check up fail as many variable are added.


			#-1 = remove link between pahtways and compounds, if each analyte was not linked to that pathway
			#>0 = if each analyte was not linked to the pathway, add dummy-variable representing an compound not present in the pathway
			if($zero == -1 || $zero > 0) {
				for my $key (keys(%analyte_maps)) {

					if((scalar(keys( %{$analyte_maps{$key}} ))) != (scalar(keys( %analyte_freq )))) {
						if($zero == -1) { delete($map_compound_meta{$key}{$temp_cmp}); }
						
						#if the pathway is not supported by all analytes, dummy-variables are being added to the matrix
						#user can decide the maximum number of dummy-variables to be added to the matrix
						if($zero > 0) {
							$zero_counter = 0; 
							for(my($k) = (scalar(keys( %{$analyte_maps{$key}} ))); $k < (scalar(keys( %analyte_freq ))); ++$k) {						
								if($zero_counter < $zero) {
									$map_compound_meta{$key}{$temp_cmp}{"CXXXXX$zero_counter"} = 1; 
								}
								++$zero_counter;
							}
						}
					}
				}
			}

			#if(scalar(keys(%all_maps)) > 0) { print "$print_line\n"; }
		}
	
		#if mode == 1, use also unknown to separate the two ends
		++${$maps}{"mapall"}{'nall'};
		++$counter;
	}

	#############################################################################################
	#calculate how many of the compounds were linked to at some pathway
	for my $key1 (keys(%map_compound_meta)) {
		%this_map_compound_meta = %{$map_compound_meta{$key1}};
		if(scalar(keys(%this_map_compound_meta)) == 0) {
			delete($map_compound_meta{$key1});
		} else {
			for my $key2 (keys(%this_map_compound_meta)) { $nfreq{$key2} = 0; }
		}
	}
	${$maps}{"mapall"}{'n'} = scalar(keys(%nfreq));
	#print STDERR "".(${$maps}{"mapall"}{'n'})."\n";

	#############################################################################################
	#for each pathway, iterate different compound combinations
	#and take either min, max or median occurrence of compounds
	for my $key (keys(%map_compound_meta)) {
		#this lists the compounds (and all their KEGGids) beloning to each pathway
		%this_map_compound_meta = %{$map_compound_meta{$key}};
		#all compounds (e.g. compound123)
		@this_map_keys		= keys(%this_map_compound_meta);
		%freqlist		= ();
		@pathlist		= ();
		@freqarray		= ();

		#the code-line below is mainly for testing as it speeds up the analysis
		if($key eq 'map01100' || $key eq 'map01110') { next; }

		&hash_iteration(\%this_map_compound_meta, 0, "", (scalar(keys(%this_map_compound_meta))), \@this_map_keys, \@pathlist);

		#pathlist contains all different KEGG-id combinations that can be created from the list of all KEGG-ids
		for(my($x) = 0; $x < @pathlist; ++$x) {
			#print STDERR "$pathlist[$x]\n";
			@num_compound = split(/\,/, $pathlist[$x]);
			if(!(exists($freqlist{scalar(@num_compound)}))) {
				$freqlist{scalar(@num_compound)} = 1;
			} else {
				$freqlist{scalar(@num_compound)} = $freqlist{scalar(@num_compound)} + 1;
			}
			push(@freqarray, scalar(@num_compound));
		}
	
		@freqarray = sort(@freqarray);
		if($kmode == 0) { $usek = $freqarray[0]; }				#use min
		if($kmode == 2) { $usek = $freqarray[scalar(@freqarray)-1]; }		#use max
		if($kmode == 1) { $usek = $freqarray[int((scalar(@freqarray)-1)/2)]; }	#use median

		#if(scalar(keys(%freqlist)) >= 2) {
		#	print STDERR "X $key\n";
		#	for my $key (keys(%freqlist)) {
		#		print STDERR "$key\t$freqlist{$key}\n";
		#	}
		#	print STDERR "$usek\n\n";
		#}

		${$maps}{$key}{'k'} = $usek;
	}
	
	for my $key (keys( %compounds_for_maps)) {
		$this_maps = "";
		for my $key2 (sort(keys( %{$compounds_for_maps{$key}} ))) {
			$this_maps .= "$key2,";
		}
		${$maps}{$key}{'compound'} = $this_maps;
	}
};
#########################################################################################################
#
sub hash_iteration {
	my($hash_ptr, $this_depth, $path, $max_depth, $key_array, $pathlist)		= @_;
	my($new_path)									= "";
	my($final_path)									= "";
	my($this_comp)									= "";
	my(@metabols)									= ();
	my(%uniqmetabols)								= ();

	$this_comp = @$key_array[$this_depth];
	for my $key (keys(%{${$hash_ptr}{$this_comp}})) {
	
		$new_path = $path."$key,";
		if($this_depth != ($max_depth-1)) { 
			&hash_iteration($hash_ptr, ($this_depth+1), $new_path, $max_depth, $key_array, $pathlist);
		} else {
			$new_path 	=~ s/,$//g;
			@metabols 	= split(/,/, $new_path);
			%uniqmetabols 	= ();
			$final_path	= "";
			for(my($i) = 0; $i < @metabols; ++$i) {
				if(!(exists($uniqmetabols{$metabols[$i]}))) {
					if(!($metabols[$i] =~ m/CXXXXX\d+/)) {
						$final_path .= "".$metabols[$i].",";
						$uniqmetabols{$metabols[$i]} = 1
					}
				}
			}
			$final_path =~ s/CXXXXX\d+,//g;	#remove dummy variable, should not be here at all, but just to be sure
			$final_path =~ s/,+$//g;
			push(@$pathlist, $final_path);
		}
	}
};
#########################################################################################################

1;
