use strict;
use warnings;

# ***************************************************************************
# *   Copyright (C) 2010 Matti Kankainen                                    *
# *   matti.kankainen@helsinki.fi                                           *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU General Public License as published by  *
# *   the Free Software Foundation; either version 2 of the License, or     *
# *   (at your option) any later version.                                   *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU General Public License for more details.                          *
# *                                                                         *
# *   You should have received a copy of the GNU General Public License     *
# *   along with this program; if not, write to the                         *
# *   Free Software Foundation, Inc.,                                       *
# *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
# ***************************************************************************/

###################################################################################################
#
sub make_html {
	my($output, $txtoutput, $kegg_figures)								= @_;
	my($line_out)											= "";
	my($is_print)											= 0;
	my($print_java)											= 0;

	open(HTML, ">$output");
	$is_print = &print_file_to_html("$txtoutput\_0", $kegg_figures, 1, 0);
	
	if($is_print == 1) {
		$print_java = 0;
	} else {
		$print_java = 1;
	}

	$is_print = &print_file_to_html("$txtoutput\_1", $kegg_figures, $print_java, 1);
	print HTML "<br>\n";
	print HTML "<dtml-var mpea_footer>\n";

	close HTML;
}
###################################################################################################
#
sub print_file_to_html {
	my($txtoutput, $kegg_figures, $print_java, $filenum)						= @_;
	my($counter)											= 0;
	my($map_counter)										= "";
	my($line)											= "";
	my(@cols)											= ();
	my($line_out)											= "";
	my($is_print)											= 0;
	my($direction)											= "";
	my(@file)											= ();
	my($resfile)											= "";
	my($mapfile)											= "";

	@file = split(/\//, $txtoutput);
	$resfile = $file[scalar(@file)-1];
	$mapfile = $resfile;
	$mapfile =~ s/enr\..*$//g;
	$mapfile .= "map.inpos1";

	if($filenum == 0) { 
		$direction = "Top down";
	} if($filenum == 1) {
		$direction = "Bottom up";
	}

	if(-e "$txtoutput") {
		$line_out = &html_header($print_java, $direction);
		print HTML "$line_out\n";

		open(IN, "$txtoutput") or die "$txtoutput not found\n";
		$counter = 1;
		while($line = <IN>) {
			$line =~ s/^\s+|\s+$//g;
			if((!($line =~ m/^#/)) && (length($line) > 0)) {
				@cols           = split(/\t/, $line);
				$cols[9]        =~ s/^#//g;
				$cols[9]        =~ s/,/ /g;
				$map_counter    = "$cols[0]\_$filenum\_$counter";

				$line_out = &html_best($map_counter, $cols[0], $cols[1], $cols[2], $cols[3], $cols[4], $cols[5], $cols[6], $cols[7], $cols[8], $counter);
				print HTML "$line_out\n";
				$line_out = &html_description($cols[0], $cols[9], $txtoutput, $kegg_figures);
				print HTML "$line_out\n</td></tr>\n\n";
				++$counter;
			}
		}
		close IN;
		print HTML "</table><br>\n";

		print HTML "<html><head><title>MPEA</title></head>\n";
		print HTML "<h2 align='center'>\n";
		print HTML "<a href='http://ekhidna.biocenter.helsinki.fi/poxo/mpea/results/tmp/$resfile'>Results.txt</a>\n";
		print HTML "<a href='http://ekhidna.biocenter.helsinki.fi/poxo/mpea/results/tmp/$mapfile'>Mappings.txt</a>\n";
		print HTML "</h2></html>\n";
		$is_print = 1;
	}
	return($is_print);
};
###################################################################################################
#
sub html_header {
	my($java, $direction)										= @_;
	my($text)											= "";

	my($java_txt) = "".qq(<html><body><script type="text/javascript" language="JavaScript">
			<!--
			function openClose(id){
				var el = document.getElementById(id);
				if(el.className == "openClass"){
					el.className = "closedClass";
				} else{
					el.className = "openClass";
				}
			}
			//-->
			</script>
                                
			<style type="text/css">
				body { font-family:monospace; }
				p {text-align:center;}
				p.h1 {font-size:16px;text-decoration:underline;}
				p.h2 {font-size:14px;text-decoration:underline;}
				.openClass { display: table-row; }
				.closedClass { display: none; }
			</style>);


	my($table)    = "".qq(<br>
			<br>
			<h2 align="center">$direction</h2>
			<TABLE border="1" align="center" cellpadding="5" cellspacing="0" width="90%">
			<thead>
			<tr>
			<th>Ind</td><th>MapID</th><th>Pathway name</th>
			<th>N (KEGG tot)</th><th>K (pathway tot)</th><th>n (with KEGG-ID)</th><th>n (all)</th>
			<th>k</th><th>p-value</th><th>perm val</th><th>p-adj (fdr)</th>
			</tr>
			</thead>
			<tfoot>
			<tr>
			<th>Ind</td><th>MapID</th><th>Pathway name</th>
			<th>N (KEGG tot)</th><th>K (pathway tot)</th><th>n (with KEGG-ID)</th><th>n (all)</th>
			<th>k</th><th>p-value</th><th>perm val</th><th>p-adj (fdr)</th>
			</tr>
			</tfoot>
			<tbody>);

	if($java == 1) { 
		$text = "$java_txt\n$table\n"; 
	} else { 
		$text = "$table\n"; 
	}
	return($text);
};
###################################################################################################
sub html_description {
	my($map, $comp, $file, $kegg_figures)								= @_;
	my($text)											= "";
	my($mapfile)											= "";
	my(@cols)											= ();

	@cols	= split(/\//, $file);
	$file	= $cols[scalar(@cols)-1];
	$mapfile = $map;
	$mapfile =~ s/^map/ko/;
	$mapfile = "$file\_$mapfile.png";
	$mapfile = "<img src=\"http://ekhidna.biocenter.helsinki.fi/poxo/mpea/results/tmp/$mapfile\"\>";

	if($map =~ m/SMP/) {
		$mapfile = "<img src=nulle\"\>";
	}

	if($kegg_figures == 0) { $mapfile = "Figure was not constructed"; }

	$text .= 	qq(<p class="h1"><font size="5"><b>$map</b></font></p>
			<table align="center" border="1" cellpadding="5" cellspacing="0" width="90%">
			<tr><th>KEGG compounds: $comp</th></tr>
			<tr align="center">
			<th>$mapfile</th>
			</tr>
                        </table></td></tr>\n);

	return($text);

}
###################################################################################################
sub html_best {
	my($map_c, $map, $name, $ntot, $ktot, $ntest, $ktest, $pval, $pperm, $fdr, $counter)			= @_; 
	my($text)											= "";
	my(@cols)											= split(/\//, $ntest);
	$cols[0] =~ s/[\(\)]//g;
	$cols[1] =~ s/[\(\)]//g;
                                
	$text = 	qq(<tr class="openClass">
			<td>$counter</td>
			<td><a href="javascript:openClose('$map_c');">$map</a></td>
			<td>$name</td>
			<td>$ntot</td>
			<td>$ktot</td>
			<td>$cols[0]</td>
			<td>$cols[1]</td>
			<td>$ktest</td>
			<td>$pval</td>
			<td>$pperm</td>
			<td>$fdr</td>
			</tr>
			<tr class="closedClass" id="$map_c">
			<td colspan="11">\n);
	return($text);
};
###################################################################################################
1;
