#!/usr/bin/perl

# ***************************************************************************
# *   Copyright (C) 2010 Matti Kankainen                                    *
# *   matti.kankainen@helsinki.fi                                           *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU General Public License as published by  *
# *   the Free Software Foundation; either version 2 of the License, or     *
# *   (at your option) any later version.                                   *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU General Public License for more details.                          *
# *                                                                         *
# *   You should have received a copy of the GNU General Public License     *
# *   along with this program; if not, write to the                         *
# *   Free Software Foundation, Inc.,                                       *
# *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
# ***************************************************************************/

use strict;
use warnings;
use FindBin qw($Bin);
use Getopt::Long;

use lib "$Bin";
use golm_mapper;
use golm_remover;
use html_code;

&main();

#perl ../mpea.pm --input_file=megaliver_data.txt --skip_mapping=0 --output=mega_liver --direction=2 --permutation=100 --gsea=1 --penalty_mode=1 --kegg_figure=1

#perl mpea.pm --input_file=DIPP/dipp_results3.mapper --skip_mapping=1 --output=testx --direction=0 --gsea=0 --penalty_mode=2 --kegg_figure=0 --permutation=20

#perl mpea.pm --input_file=all_withranks.txt --skip_mapping=1 --output=ad_retest --direction=2 --gsea=1 --penalty_mode=1 --kegg_figure=0 --permutation=10

#########################################################################################################
#read parameters
sub main {
	my($input_file)						= "";
	my($column)						= "";
	my($column_width)					= "";
	my($output)						= "";
	my($skip_mapping)					= "";

	my($no_orginfo)						= "";
	my($organism)						= "";
	my($gsea)						= "";
	my($permutation)					= "";
	my($list_size)						= "";
	my($penalty_mode)					= "";
	my($kselection)						= "";
	my($kegg_mapfile)					= "";
	my($kegg_cpdfile)					= "";
	my($kegg_userbck)					= "";
	my($direction)						= "";
	my($kegg_figures)					= "";
	my($enrich_output)					= "";
	my($html)						= "";
	my(@golm_param)						= ();
	my(@temp_output)					= ();
	my($line)						= "";
	my(@cols)						= ();
	my(@mapper_data)					= ();
	my(%mapper_ranks)					= ();
	my(%real_ranks)						= ();
	my($enrich_input)					= "";
	my($counter)						= 0;
	my($rank)						= 0;
	my($maxcount)						= 0;
	my($map_counter)					= "";
	my($line_out)						= "";
	my($mink)						= 0;

	########################################################################################################################
       	#define inputs and their types 
	#parser commandline options
	my %opt = ();
	my %legend = (
		"input_file=s"		=>      "spectral file or file for which golm-mapping has been done <string>",
		"column=s"   		=>      "name of the column <VAR5>",
		"column_width=i"	=>      "RI width to be used <25>",
		"output=s"		=>      "name of the output file <string> (temp)",
		"skip_mapping=i"	=>	"If set to 1, no spectra is not map to GOLM-db or not <0|1> (1)",
		"html=i"		=>	"If set to 1, html-output file is created <0|1> (0)",
		
		"dotproduct=f"		=>	"dotproduct (0.05)",
		"euclideandist=f"	=>	"euclideanDistance (0.05)",
		"hammingdist=f"		=>	"hammingDistance (500)",
		"jaccarddist=f"		=>	"jaccardDistance (0.9)",
		"binarydist=f"		=>	"s12GowerLegendreDistance (0.9)",

		"organism=s"		=>	"organism to be included to the analysis <regexp> (HSA)",
		"gsea=i"		=>	"If set to 1, enrichment is calculated iteratively <0|1> (1)",
		"permutation=i"		=>	"number of permutations to be done <0-100> (100)",
		"direction=i"		=>	"is analysis done in both=2, up to down=1 or down to up=2 direction <0|1|2> (2)",
		"list_size=i"		=>	"number of features to be analysed <0-1000> (0=all)",
		"penalty_mode=i"	=>	"how to deal pathway inconsitensies: 0=no, 1=remove incosistent IDs, 2-X=add n dummy variables <0|1|2> (2)",
		"kselection=i"		=>	"k-choosing method (0=minumum k, 1=medium k, 2=maximum k) <0|1|2> (2)",
		"mink=i"		=>	"minimum number of compounds that a pathway must hold to be analyzed (3)",
		"kegg_mapfile=s"	=>	"name of the file having KEGG map-names <string> (KEGG/map_title.tab)",
		"kegg_cpdfile=s"	=>	"name of the file having KEGG compound informatio<string> (KEGG/ec_cpd_map.tab)",
		"kegg_figures=i"	=>	"If set to 1, KEGG figures are drawns <0|1> (1)",
		"kegg_userbck=s"	=>	"file describing a set of KEGG-compounds to be used for background distribution",
	);
	
	&GetOptions(\%opt, "help!", keys %legend);
	
	if(scalar(scalar keys %opt) == 0 || exists($opt{'help'})) {
                print "input_file=s      =>    spectral file or file for which golm-mapping has been done <string>\n";
                print "column=s          =>    name of the column <string> (VAR5)\n";
                print "column_width=i    =>    RI width to be used <int> (25)\n";
                print "output=s          =>    name of the output file <string>\n";
                print "skip_mapping=i    =>    if set to 1, spectra is not map to GOLM-db or not <0|1>\n";
		print "html=i            =>    if set to 1, html-output file is created <0|1> (0)\n\n";

                print "dotproduct=i      =>    Dotproduct cutoff to accept spectra-matches on GOLMdb <double> (<=0.05)\n";
                print "euclideandist=i   =>    Euclidean distance cutoff to accept spectra-matches on GOLMdb <double> (<=0.05)\n";
                print "hammingdist=i     =>    Hamming distance cutoff to accept spectra-matches on GOLMdb <int> (<=500)\n";
                print "jaccarddist=i     =>    Jaccard distance cutoff to accept spectra-matches on GOLMdb <double> (<=0.9)\n";
                print "binarydist=i      =>    s12GowerLegendreDistance distance cutoff to accept spectra-matches on GOLMdb <double> (<=0.9)\n\n";

                print "organism=s        =>    organism to be included to the analysis <regexp> (HSA). Type all to include every compound\n";
		print "no_orginfo=i      =>    include (1) or not (0) those compounds that have not been associated to any organism <0|1> (0)\n";
                print "gsea=i            =>    if set to 1, enrichment is calculated iteratively <0|1> (1)\n";
                print "permutation=i     =>    number of permutations to be done <unsigned int> (100)\n";
		print "direction=i       =>    is analysis done in both=2, up to down=0 or down to up=1 <0|1|2> (2)\n";
                print "list_size=i       =>    number of features to be analysed <0=all|unsigned int> (0)\n";
                print "penalty_mode=i    =>    pathway inconsistency method: 0=no, -1=remove incosistent IDs, 1-int=n dummy variables <-1|0|1-n> (1)\n";
                print "kselection=i      =>    k-choosing method, 0=minumum k, 1=medium k, 2=maximum k <0|1|2> (1)\n";
		print "mink=i            =>    pathway is analyzed at some rank-position, if at least this number of compounds was assigned to it (3)\n";
                print "kegg_mapfile=s    =>    name of the file having KEGG map-names <string> (KEGG/map_title.tab)\n";
		print "kegg_cpdfile=s    =>    name of the file having KEGG compound informatio<string> (KEGG/cpd_map_org.tab)\n";
		print "kegg_figures=i    =>    if set to 1, KEGG figures are drawns <0|1> (1)\n";
		print "kegg_userbck=s    =>    file describing a set of KEGG-compounds to be included to the background distribution\n\n";
	}

	if(exists($opt{'input_file'}))	 	{ $input_file	= $opt{'input_file'};	} else { die "--input_file <string> not given\n"; }
	if(exists($opt{'skip_mapping'}))	{ $skip_mapping	= $opt{'skip_mapping'};	} else { die "--skip_mapping <0|1> not given\n"; }
	
	if(exists($opt{'html'}))		{ $html		= $opt{'html'};		} else { $html = 0; }
	if(exists($opt{'column'}))		{ $column	= $opt{'column'};	} else { $column = "VAR5"; }				#
	if(exists($opt{'column_width'}))	{ $column_width	= $opt{'column_width'};	} else { $column_width = 25; }				#
	if(exists($opt{'output'}))		{ $output	= $opt{'output'};	} else { $output = "null"; }				#


        if(exists($opt{'dotproduct'}))		{ $golm_param[0] = $opt{'dotproduct'};		} else { $golm_param[0] = 0.05; }
        if(exists($opt{'euclideandist'}))	{ $golm_param[1] = $opt{'euclideandist'};	} else { $golm_param[1] = 0.05; }
        if(exists($opt{'hammingdist'}))		{ $golm_param[2] = $opt{'hammingdist'};		} else { $golm_param[2] = 500; }
        if(exists($opt{'jaccarddist'}))		{ $golm_param[3] = $opt{'jaccarddist'};		} else { $golm_param[3] = 0.9; }
        if(exists($opt{'binarydist'}))		{ $golm_param[4] = $opt{'binarydist'};		} else { $golm_param[4] = 0.9; }

	if(exists($opt{'organism'}))		{ $organism	= $opt{'organism'};	} else { $organism = "HSA"; }				#
	if(exists($opt{'no_orginfo'}))		{ $no_orginfo	= $opt{'no_orginfo'};	} else { $no_orginfo = "0"; }
	if(exists($opt{'gsea'}))		{ $gsea		= $opt{'gsea'};		} else { $gsea = "1"; }					#
	if(exists($opt{'permutation'}))		{ $permutation	= $opt{'permutation'};	} else { $permutation = "100"; }			#
	if(exists($opt{'direction'}))		{ $direction	= $opt{'direction'};	} else { $direction = 2; }				#
	if(exists($opt{'list_size'}))		{ $list_size	= $opt{'list_size'};	} else { $list_size = 0; }				#
	if(exists($opt{'penalty_mode'}))	{ $penalty_mode	= $opt{'penalty_mode'};	} else { $penalty_mode = 1; }				#
	if(exists($opt{'kselection'}))		{ $kselection	= $opt{'kselection'};	} else { $kselection = 1; }				#
	if(exists($opt{'mink'}))		{ $mink		= $opt{'mink'};		} else { $mink = 3; }					#
	if(exists($opt{'kegg_mapfile'}))	{ $kegg_mapfile	= $opt{'kegg_mapfile'};	} else { $kegg_mapfile = "$Bin/KEGG/map_title.tab"; }   #
	if(exists($opt{'kegg_cpdfile'}))	{ $kegg_cpdfile	= $opt{'kegg_cpdfile'};	} else { $kegg_cpdfile = "$Bin/KEGG/cpd_map_org.tab"; } #
	if(exists($opt{'kegg_figures'}))	{ $kegg_figures	= $opt{'kegg_figures'};	} else { $kegg_figures = 1; }
	if(exists($opt{'kegg_userbck'}))	{ $kegg_userbck = $opt{'kegg_userbck'};	} else { $kegg_userbck = "null"; }

	#force direction to be zero or one
	if($direction > 2) { $direction = 1; }
	if($direction < 0) { $direction = 0; }

	#if no gsea is performed, bottom-up and up-down are the same and there is not idea to calculate both
	if($gsea == 0) { $direction = 0; $permutation = 0;}

	#alphabets from which random names are drawn
	my(@alpha)	= ("a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z");
        my($rnd_file)	= "";

	#######################################################################################################################
	#generate random prefix that is attached to every file
	for(my($i) = 0; $i < 10; ++$i) {
		$rnd_file .= $alpha[int(rand(scalar(@alpha)))];
	}
	if($output eq 'null') {
		$output = "temp".$rnd_file;
	}

	#######################################################################################################################
	#do mapping
	if($skip_mapping == 0) {
		&golm_mapper($input_file, $column, $column_width, "$output.map", \@golm_param);
		$enrich_input	= "$output.map";
	} else {
		#if(!(-e "$output.map")) { die "$output.map is not found\n"; }

		if(-e("$output.map")) {
			$enrich_input   = "$output.map";	#used in production
		} else {
			$enrich_input	= $input_file;		#used in web-server
		}
	}
        
	$enrich_output  = "$output.enr";
	if(-e("$enrich_output\_0")) { unlink("$enrich_output\_0"); }
	if(-e("$enrich_output\_1")) { unlink("$enrich_output\_1"); }

	#######################################################################################################################
	#read ranks from the original spectra-file and store them 
	$counter = 0;
	open(IN, $input_file) or die "Read rank failed. $input_file not found\n";
	while($line = <IN>) {
		$line =~ s/^\s+|\s+$//g;
		if( (!($line =~ m/^#/)) && (length($line) > 0)) {
			@cols = split(/\t/, $line);
			for(my($i) = 3; $i < @cols; ++$i) {
				$real_ranks{$counter}{$i} = $cols[$i];
			}
			++$counter;
		}
	}
	close IN;
	$maxcount = $counter;

	#######################################################################################################################
	#read data from *.map file, remove extra lines and sort data by the given ranks (order should be the same)
	$counter = 0;
	$rank	 = 0;
	open(IN, $enrich_input) or die "Read map file failed. $enrich_input not found\n";
	while($line = <IN>) {
		if(!($line =~ m/^\s+#/)) {
			push(@mapper_data, $line);
			if(exists($real_ranks{$counter})) {
				for my $pos (keys(%{$real_ranks{$counter}})) {
					$rank = $real_ranks{$counter}{$pos};
					$mapper_ranks{"pos$pos"}{$rank} = $counter;
				}
			} else {
				$mapper_ranks{'pos1'}{$counter} = $counter;
			}
			++$counter;
		}
	}
	close IN;

	#######################################################################################################################
	#create temp file and do enrichment analysis
	open(COMB_OUT, ">$enrich_output") or die "$enrich_output was not opened from comb output\n";
	for my $pos (sort(keys(%mapper_ranks))) {	
		open(OUT, ">$enrich_input.in$pos") or die "$output not found\n";
		for my $this_rank (sort { $a <=> $b; } ( keys(%{$mapper_ranks{$pos}} ))) {
			if(exists($mapper_ranks{$pos}{$this_rank})) {
				$counter = $mapper_ranks{$pos}{$this_rank};
				print OUT "$mapper_data[$counter]";
			} else {
				print STDERR "$pos $this_rank is not listed\n";
			}
		}
		close OUT;

		if($direction == 2) {
			&kegg_enrichment($kegg_mapfile, $kegg_cpdfile, $organism, "$enrich_input.in$pos", "$enrich_output.in$pos\_0", 
			 	 $gsea, $list_size, $kselection, $penalty_mode, 0, ($permutation+1), $kegg_figures, $pos, $mink, $kegg_userbck, $no_orginfo);

			&kegg_enrichment($kegg_mapfile, $kegg_cpdfile, $organism, "$enrich_input.in$pos", "$enrich_output.in$pos\_1",
				 $gsea, $list_size, $kselection, $penalty_mode, 1, ($permutation+1), $kegg_figures, $pos, $mink, $kegg_userbck, $no_orginfo);
		} else {
			&kegg_enrichment($kegg_mapfile, $kegg_cpdfile, $organism, "$enrich_input.in$pos", "$enrich_output.in$pos\_$direction",
				 $gsea, $list_size, $kselection, $penalty_mode, $direction, ($permutation+1), $kegg_figures, $pos, $mink, $kegg_userbck, $no_orginfo);
		}

		if($html == 1) {
			&make_html($output, "$enrich_output.in$pos", $kegg_figures);
			open(CHECK, ">$input_file.checkme");
			print CHECK "done\n";
			close CHECK;
		}
		
		&print_comb_results("$enrich_output.in$pos\_0");
		&print_comb_results("$enrich_output.in$pos\_1");
	}
	close COMB_OUT;

};
#########################################################################################################
#
sub print_comb_results {
	my($file)						= @_;
	my(@temp_output)					= ();
	my($line)						= "";

	if(-e($file)) {
		open(THIS_IN, $file);
		while($line = <THIS_IN>) {
			print COMB_OUT "$line";
		}
		close THIS_IN;	
	}
};
#########################################################################################################


