#!/usr/bin/perl

use strict;
use warnings;

if(@ARGV < 2) { die "usage: $0 file file \nperl $0 KEGG/compound KEGG/ec_cpd_map.tab\n"; }
&main($ARGV[0], $ARGV[1]);

#########################################################################################################
#read parameters
sub main {
	my($file, $ec_map_file)			= @_;
	my($line)				= "";
	my(@entry)				= ();
	my($counter)				= 0;
	my(@cols)				= ();
	my(%ec_hash)				= ();
	my($generig_ec1)			= "";
	my($generig_ec2)			= "";
	my($generig_ec3)			= "";
	my($org_list)				= "";
	my(@org_array)				= ();

	open(IN, $ec_map_file) or die "$ec_map_file not found\n";
	while($line = <IN>) {
		$line    	=~ s/\s+$//g;
		@cols	 	= split(/\t/, $line);
		$cols[0]	=~ s/^EC\s+//g;
		$cols[0] 	=~ m/^\d+\./;
		$generig_ec1	= "$&-.-.-";
		$cols[0]        =~ m/^\d+\.\d+\./;
		$generig_ec2   = "$&-.-";
		$cols[0]        =~ m/^\d+\.\d+\.\d+\./;
		$generig_ec3   = "$&-";

		if(@cols > 1) {
			for(my($i) = 1; $i < @cols; ++$i) {
				if((!($cols[$i] =~ m/map/)) && (!($cols[$i] =~ m/C\d{3,10}/))) {
					@org_array = split(/\,/, $cols[$i]);
					for(my($j) = 0; $j < @org_array; ++$j) {
						$ec_hash{$cols[0]}{$org_array[$j]} = 0;
						$ec_hash{$generig_ec1}{$org_array[$j]} = 0;
						$ec_hash{$generig_ec2}{$org_array[$j]} = 0;
						$ec_hash{$generig_ec3}{$org_array[$j]} = 0;
					}
				}
			}
		}

	}
	close IN;

	open(IN, $file) or die "$file not found\n";
	#reads the file untile the end-char is detected
	while($line = <IN>) {
		$line 	=~ s/\s+$//g;
		#end of entry mark
		if($line =~ m/\/\/\//g) {
			++$counter;
			&print_me(\@entry, \%ec_hash);
			@entry	= ();
	
			#if($counter ==10) { die; }
		} else {
			push(@entry, $line);
		}
	}
	close IN;

};
#########################################################################################################
#
sub print_me {
	my($entry, $ec_pointer)			= @_;
	my($print_results)			= "";
	my(%entry_hash)				= ();
	my(@cols)				= ();
	my($print_me)				= 1;
	my($print_line)				= "";
	my(@enzymes)				= ();
	my(%uniq_enzymes)			= ();
	my($enzyme_line)			= "";

	$print_results = &print_entry($entry, "ENTRY");
	@cols = split(/\|/, $print_results);
	for(my($i) = 0; $i < @cols; ++$i) {
		$cols[$i] =~ m/C[\d]{0,6}/;
		$entry_hash{$&} = "";
	}

	$print_results = &print_entry($entry, "ENZYME");
	$print_results =~ s/^\s+|\s+$//g;
	$print_results =~ s/\s+/,/g;
	$print_results =~ s/\(\D?\)//g;
	$print_results =~ s/\|+/,/g;

	@enzymes       = split(/\,/, $print_results);
	for(my($i) = 0; $i < @enzymes; ++$i) {
		for my $key (keys( %{${$ec_pointer}{$enzymes[$i]}} ) ) {
			$uniq_enzymes{$key} = 0;
		}
	}
	$enzyme_line = "";
	for my $key (keys(%uniq_enzymes)) {
		$enzyme_line .= "$key,";
	}
	$enzyme_line =~ s/\,+$//g;

	for my $key (keys(%entry_hash)) {
		$entry_hash{$key} .= "$print_results\t";
		$entry_hash{$key} .= "$enzyme_line\t";
	}

        $print_results = &print_entry($entry, "PATHWAY");
        $print_results =~ s/^.*?ko0/map0/g;
        $print_results =~ s/\s+.*?ko0/,map0/g;
        $print_results =~ s/\s+.*$//;
        for my $key (keys(%entry_hash)) {
                $entry_hash{$key} .= "$print_results";
        }

	for my $key (keys(%entry_hash)) {
		$entry_hash{$key} =~ s/,\s*$//g;
		$print_line = "$key\t$entry_hash{$key}";
		print "$print_line\n";
	}
};
#########################################################################################################
#
sub print_entry {
	my($all_entries, $this_entry)		= @_;
	my($line)				= "";
	my($print_data)				= "";
	my($counter)				= -1;

	while(($counter < @$all_entries) && (!($line =~ m/^$this_entry/))) {
		++$counter;
		$line	= @$all_entries[$counter];
	}

	while(($counter < @$all_entries) && ($line =~ m/^$this_entry/ || $line =~ m/^\s+/)) {
		$line =~ s/^$this_entry\s+//g;
		$line =~ s/^\s+//g;
		$print_data .= "|$line";

		++$counter;
		$line = @$all_entries[$counter];
	}

	$print_data =~ s/^?\|//;	
	$print_data =~ s/^\s+|\s+$//g;
	return($print_data);
}
#########################################################################################################

