#!/usr/bin/perl

use strict;
use warnings;

my($org) = "null";

if(@ARGV < 1) { die "usage: $0 file organism\nperl $0 KEGG/enzyme 'HS?' (HSA|XXX|TPT)\n"; }
if(defined($ARGV[1])) { $org = $ARGV[1]; }
&main($ARGV[0], $org);

#########################################################################################################
#read parameters
sub main {
	my($file, $org)				= @_;
	my($line)				= "";
	my(@entry)				= ();
	my($counter)				= 0;

	open(IN, $file) or die "$file not found\n";

	#reads the file untile the end-char is detected
	while($line = <IN>) {
		$line 	=~ s/\s+$//g;
		#end of entry mark
		if($line =~ m/\/\/\//g) {
			++$counter;
			&print_me(\@entry, $org);
			@entry	= ();
	
			#if($counter == 6) { die; }
		} else {
			push(@entry, $line);
		}
	}
};
#########################################################################################################
#
sub print_me {
	my($entry, $org)			= @_;
	my($print_results)			= "";
	my(%entry_hash)				= ();
	my(@cols)				= ();
	my($print_me)				= 1;
	my($print_line)				= "";
	my($temp_print_res)			= "";
	my($temp_comps)				= "";

	#######################################################
	#ENTRY
	$print_results = &print_entry($entry, "ENTRY");
	@cols = split(/\|/, $print_results);
	for(my($i) = 0; $i < @cols; ++$i) {
		$cols[$i] =~ m/EC\s+[\d\.]{0,12}/;
		$entry_hash{$&} = "";
	}

	#######################################################
        #SUBSTRATE / PRODUCT
	$print_results  = &print_entry($entry, "SUBSTRATE");
	$print_results .= "|";
	$print_results .= &print_entry($entry, "PRODUCT");
	$temp_print_res = "";
	while($print_results =~ m/\[CPD:C\d+?\]/ig) {
		$temp_print_res .= "$&,";
	}
	$temp_print_res =~ s/\,+$//g;
	$temp_print_res =~ s/\[CPD://g;
	$temp_print_res =~ s/\]//g;
	for my $key (keys(%entry_hash)) {
		$entry_hash{$key} = "$temp_print_res\t";
	}
	
	#######################################################
	#PATHWAY
        $print_results = &print_entry($entry, "PATHWAY");
        $print_results =~ s/^.*?ec0/map0/g;
        $print_results =~ s/\s+.*?ec0/,map0/g;
        $print_results =~ s/\s+.*$//;
        for my $key (keys(%entry_hash)) {
                $entry_hash{$key} .= "$print_results\t";
        }

	#######################################################
        #GENES
	if($org ne "null") { $print_me = 0; }
	$print_results = &print_entry($entry, "GENES");
	@cols = split(/\|/, $print_results);
	for(my($i) = 0; $i < @cols; ++$i) {
		if($cols[$i] =~ m/\D\D\D:/g) {
			for my $key (keys(%entry_hash)) {
				$entry_hash{$key} .= "".substr($&,0,3).",";

				if($org ne "null") {
					if(substr($&,0,3) =~ m/$org/i) {
						$print_me = 1;
					}
				}
			}
		}
	}

	if($print_me == 1) {
		for my $key (keys(%entry_hash)) {
			$entry_hash{$key} =~ s/,$//g;
			$print_line = "$key\t$entry_hash{$key}";
			print "$print_line\n";
		}
	}
};
#########################################################################################################
#
sub print_entry {
	my($all_entries, $this_entry)		= @_;
	my($line)				= "";
	my($print_data)				= "";
	my($counter)				= -1;

	while(($counter < @$all_entries) && (!($line =~ m/^$this_entry/))) {
		++$counter;
		$line	= @$all_entries[$counter];
	}

	while(($counter < @$all_entries) && ($line =~ m/^$this_entry/ || $line =~ m/^\s+/)) {
		$line =~ s/^$this_entry\s+//g;
		$line =~ s/^\s+//g;
		$print_data .= "|$line";

		++$counter;
		$line = @$all_entries[$counter];
	}

	$print_data =~ s/^?\|//;	
	$print_data =~ s/^\s+|\s+$//g;
	return($print_data);
}
#########################################################################################################

